# -*- coding:Utf-8 -*-

from pyGEM import *
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import *
class EntProd :

##############################################
#Definition des parametre                    #
##############################################
	AlmostZero = 10**-9
	milliNa =  6.02*10**20
	V  = 6.5*10**-16

	
	#Enzymatic parameters
	KcatB = 600   # min^-1
	KcatC = 173      # min^-1 #Fixed
	KcatA = 92.5*60    # min^-1
	KmB   = 14.7*10**-3        # mM
	KmC   = 14*10**-3       # mM #Fixed
	KmA   = 300*10**-3     # mM #Fixed

	#production parameters
	vE1 = 10**-9 #E1 production rate
	
	#kinetic parameters
	kT    = 2.0#/(microNa*V) #Transciption rate

	p     = 10**-1 #permeabilite
	kfur  = 10**-2
	kp    =  0.5/(milliNa*V) #min^-1Translation rate
	Pfur  = 10**1# fur production parameter
	E1init   = 1.0

	#Concentration parameters (mM)
	Fur0 = 1.0*10**-2#fur production value 
	Kfer = 0.394*10**-3 #Fixed

	#Degredation parameters
	Dfur = 10**(-3)
	Dfer = 10**(-3)
	Denz = 10**(-3)
	Dff = 10**(-3)
	Dmrna = 0.14/(-log(AlmostZero))

	#Numbers o5f plasmid and fur binding sites
	Nbrpla1 = 15.#Fixed
	Nbrpla2 = 300.#Fixed

	#Déclaration des Fonctions Logistiques:
	Ki1   = 6.3*10**-5
	Ki2   = 10
	kf    = 5.109350526252892*10**-6 #taux de fixation #Fixed
	M = 40000



######################################################################
######################################################################
	
	

	def __init__(self,Ferext,time,step) :
		#Variable global
		self.Time = time
		self.Step = step

		#Déclaration des variables d'états :
		self.FerExt = self.Time*[Ferext]
		self.Fer = self.Time*[Ferext]
		self.Fur = self.Time*[0.0]
		self.FurFer = self.Time*[0.0]
		self.FBS = self.Time*[0.0]
		self.pLac = self.M*[1.0]
		self.FBS = self.M*[1.0]
		self.mRNA = self.Time*[0.0]
		self.E1 = self.Time*[self.E1init]
		self.E2 = self.Time*[0]
		self.E3 = self.Time*[0]
		self.E4 = self.Time*[0]
		self.Enz = self.Time*[0]
		self.Ent = self.Time*[0]


##################################################################
# Partie Régulation de la production d'enterobactine             #
##################################################################
	
	def exprFer (self,t):
		return - self.kfur*self.Fur[t]*self.Fer[t] + self.Dff*self.FurFer[t]
		# return (self.p*self.FerExt[t] - self.kfur*self.Fur[t]*self.Fer[t] + self.Dff*self.FurFer[t])
		
	def exprFur (self,t):
		return (-self.kfur*self.Fur[t]*self.Fer[t] + self.Fur0  + self.Dff*self.FurFer[t] )
 
	def exprFurFer (self,t):
		return (self.kfur*self.Fur[t]*self.Fer[t] - self.Dff*self.FurFer[t] 
			- (self.FBS[trunc(self.FurFer[t]*100)] - self.FBS[trunc(self.FurFer[t]*100)])/(self.milliNa*self.V))

	def exprFBS(self,x):
		return self.Ki1/self.kf*self.FBS[x]*(1-self.FBS[x]/self.Nbrpla1)
		
	def exprpLac(self,x):
		return self.Ki2*self.pLac[x]*(1-self.pLac[x]/self.Nbrpla2)

	def exprmRNA (self,t):
		return (self.kT*self.pLac[trunc(self.FBS[trunc(self.FurFer[t]*100)]*100)] - self.Dmrna*self.mRNA[t])
	
	def exprEnz (self,t):
		return (self.kp*self.mRNA[t]-self.Denz*self.Enz[t])

#####################################################################
#####################################################################


#############################################
# Model de la synthése de Enterobactine     #
#############################################

	def vitesse1(self,t) :
		return (self.KcatC*self.Enz[t]*self.E1[t])/self.KmC
	
	def vitesse2(self,t) :
		return (self.KcatB*self.Enz[t]*self.E2[t])/self.KmB

	def vitesse3(self,t) :
		return (self.KcatA*self.Enz[t]*self.E3[t])/self.KmA

	def vitesse4(self,t) :
		return (self.KcatB*self.Enz[t]*self.E4[t])/self.KmB

	def exprE1 (self,t):
		if(self.vE1- self.vitesse1(t) > 0):
			return (self.vE1 - self.vitesse1(t)) 
		else:
			return 0
	def exprE2 (self,t):
		if(self.vitesse1(t) - self.vitesse2(t)> 0):
			return (self.vitesse1(t) - self.vitesse2(t)) 
		else:
			return 0
	def exprE3 (self,t):
		if(self.vitesse2(t) - self.vitesse3(t) > 0):
			return (self.vitesse2(t) - self.vitesse3(t))
		else:
			return 0
	def exprE4 (self,t):
		if(self.vitesse3(t) - 3*self.vitesse4(t) > 0) :
			return (self.vitesse3(t) - 3*self.vitesse4(t))
		else :
			return 0

	def exprEnt (self,t):
		return (self.vitesse4(t)) 



	def Proceed(self) :
		for i in range(0,self.M-1):
			self.pLac[i+1] = self.Step/100*self.exprpLac(i) + self.pLac[i]
			self.FBS[i+1] = self.Step/100*self.exprFBS(i) + self.FBS[i]
		for i in range(0,self.Time-1) :
			self.Fer[i+1] = self.Step*self.exprFer(i) + self.Fer[i]
			self.Fur[i+1] = self.Step*self.exprFur(i) + self.Fur[i]
			self.FurFer[i+1] = self.Step*self.exprFurFer(i) + self.FurFer[i]
			self.mRNA[i+1] = self.Step*self.exprmRNA(i) + self.mRNA[i]
			self.Enz[i+1] = self.Step*self.exprEnz(i) + self.Enz[i]
			self.E1[i+1] = self.Step*self.exprE1(i) + self.E1[i]
			self.E2[i+1] = self.Step*self.exprE2(i) + self.E2[i]
			self.E3[i+1] = self.Step*self.exprE3(i) + self.E3[i]
			self.E4[i+1] = self.Step*self.exprE4(i) + self.E4[i]

			self.Ent[i+1] = self.Step*self.exprEnt(i) + self.Ent[i]


			
	




tt = 7
prodfer = 0.1
it = 10.
ini = 10.
time = 80000
step = 0.1

Fer = tt*[10**-1]
x = time*[0]

Model = tt*[0]
F = tt*[0]
S = tt*[0]
y = tt*[0]


for i in range(0,tt):
	Model[i] = EntProd(ini/it**i,time,step)
	Model[i].Proceed()
	F[i] = Model[i].Enz
	S[i] = time*[6]
 	y[i] = time*[log(ini/it**i,10)]

Model = EntProd(1,time,step)
Model.Proceed()

E = time*[Model.Ent[0]/Model.Ent[time-1]]
Z = time*[Model.Enz[0]/Model.Enz[time-1]]
F = time*[Model.Fer[0]/Model.Fer[0]]

for i in range(1,time) :
	E[i] = Model.Ent[i]/Model.Ent[time-1]
	Z[i] = Model.Enz[i]/Model.Enz[time-1]
	F[i] = Model.Fer[i]/Model.Fer[0]

for i in range(0,time-1) :
	x[i+1] = x[i] + step

# fig = plt.figure("Ki1 10^-5 Ki2 10")
# ax = fig.add_subplot(111, projection='3d')
# ax.plot_wireframe(x,y, S, rstride =20000 ,cstride=20000)

# ax.plot_surface(x,y, F ,cmap = cm.coolwarm,  antialiased=False,rstride = 1, cstride=1000,shade = False)
# plt.xlabel("Time (s)")
# plt.ylabel("Iron (mM) (log scale)")
# ax.zaxis.set_label_text("Enterobactin (mM)")
# fig1 = plt.figure("LacI")
# ax1 = fig1.add_subplot(111, projection='3d')

# ax1.plot_wireframe(x,Fer, S ,cstride=200)


# fig2 = plt.figure("LacO")
# ax2 = fig2.add_subplot(111, projection='3d')

# ax2.plot_wireframe(x,Fer, A ,cstride=200)




fig2D = plt.figure()
ax3 = fig2D.add_subplot(111)
ax3.plot(x,E,label = 'Enterobactin',color = 'k',linewidth=2)
ax3.plot(x,Z,label = 'Enzyme',color = 'r',linewidth=2)
ax3.plot(x,F,label = 'Iron',color = 'b',linewidth=2)

plt.xlabel("Time (min)")
plt.ylabel("Proportion")
plt.legend().draggable()

plt.show()




